import pandas as pd
import altair as alt
from dateutil.parser import parse


def plot_summary(
    df,
    var="location",
    val="new_cases",
    fun="sum",
    date_from=None,
    date_to=None,
    top_n=5,
):
    """Generate summary plot
    Create a horizontal bar chart summarising a specified variable and value
    within a time period

    Parameters
    ----------
    df  : Pandas dataframe
        Pandas dataframe of the selected covid data from get_data()
    var : str, optional
        Qualitative values to segment data. Must be a categorical variable.
        Also known as a 'dimension'. By default 'location'
    val : str, optional
        Quantitative values to be aggregated. Must be numeric variable.
        Also known as a 'measure'. By default 'new_cases'
    fun : str, optional
        Aggregation function for val, by default 'sum'
    date_from : str, optional
        Start date of the data range with format 'YYYY-MM-DD'. By default 'None' is used to represent 7 days prior to today's date
    date_to : str, optional
        End date of data range with format 'YYYY-MM-DD'. By default 'None' is used to represent today's date
    top_n : int, optional
        Specify number of qualitative values to show, by default 5

    Returns
    -------
    altair.Chart
        Altair bar plot for the specified variables and period

    Example
    -------
    >>> plot_summary(df, var="location", var="new_cases", fun="sum", date_from="2022-01-01", date_to="2022-01-15", top_n=10)
    """
    # init dates if None
    if date_from is None:
        date_from = (
            pd.to_datetime("today").normalize() - pd.to_timedelta(7, unit="d")
        ).strftime("%Y-%m-%d")

    if date_to is None:
        date_to = pd.to_datetime("today").normalize().strftime("%Y-%m-%d")

    # Exception Handling
    if not isinstance(df, pd.DataFrame):
        raise FileNotFoundError("Data not found! There may be a problem with data URL.")

    if not isinstance(var, str):
        raise TypeError("var needs to be of str type!")

    if not isinstance(val, str):
        raise TypeError("val needs to be of str type!")

    if not isinstance(fun, str):
        raise TypeError("fun needs to be of str type!")

    if df[var].dtypes.kind != "O":
        raise TypeError("var needs to be a categorical variable!")

    if df[val].dtypes.kind == "O":
        raise TypeError("val needs to be a numeric variable!")

    if not isinstance(top_n, int) or top_n <= 0:
        raise ValueError("top_n must be an integer bigger than zero")

    if pd.to_datetime(date_to) < pd.to_datetime(date_from):
        raise ValueError(
            "Invalid values: date_from should be smaller or equal to date_to (or today's date if date_to is not specified)."
        )
    if pd.to_datetime(date_to) > pd.to_datetime("today").normalize():
        raise ValueError("Invalid values: date_to should be smaller or equal to today.")

    # Parse date, else raise ValueError
    date_from = parse(date_from)
    date_to = parse(date_to)

    # Convert 'date' to date format
    df["date"] = pd.to_datetime(df["date"])

    # Filter by date
    df = df.query("date >= @date_from & date <= @date_to")

    # Remove aggregated locations
    df = df[~df["iso_code"].str.startswith("OWID")]

    # Aggregation
    df_plot = df.groupby(var).agg({val: fun})[val].nlargest(top_n)
    df_plot = df_plot.to_frame().reset_index()

    y_lab = var.replace("_", " ").title()
    x_lab = val.replace("_", " ").title()
    if date_from == date_to:
        subtitle = f"from {date_from.strftime('%Y-%m-%d')}"
    else:
        subtitle = (
            f"from {date_from.strftime('%Y-%m-%d')} to {date_to.strftime('%Y-%m-%d')}"
        )
    title = alt.TitleParams(f"Top {top_n} {y_lab} by {x_lab}", subtitle=[subtitle])

    return (
        alt.Chart(df_plot, title=title)
        .mark_bar()
        .encode(
            y=alt.Y(var, sort="x", title=y_lab),
            x=alt.X(val, title=x_lab),
            color=alt.Color(var, legend=None),
        )
    )



# File Name: get_data.py
# Author: Thomas Siu

import pandas as pd
from datetime import datetime


def get_data(date_from=None, date_to=None, location=None):
    """Get covid data
    Retrieve covid data in pandas dataframe format
    with the time periods and countries provided.

    Parameters
    ----------
    date_from : str, optional
        Start date of the data range with format 'YYYY-MM-DD'.
        By default 'None' is used to represent 7 days prior to today's date
    date_to : str, optional
        End date of data range with format 'YYYY-MM-DD'.
        By default 'None' is used to represent today's date
    location : list, optional
        List of target country names.
        By default 'None' is used for all countries.

    Returns
    -------
    pandas.DataFrame
        Pandas dataframe of the selected covid data.

    Examples
    --------
    >>> get_data(date_from="2022-01-01",
                date_to="2022-01-07",
                location=["Canada", "China"])
    """
    query = "@date_from <= date <= @date_to"
    url = "https://covid.ourworldindata.org/data/owid-covid-data.csv"

    if date_from is None:
        date_from = (
            pd.to_datetime("today").normalize() - pd.to_timedelta(7, unit="d")
        ).strftime("%Y-%m-%d")

    if date_to is None:
        date_to = pd.to_datetime("today").normalize().strftime("%Y-%m-%d")

    try:
        date_from != datetime.strptime(date_from, "%Y-%m-%d").strftime("%Y-%m-%d")
        #            raise ValueError
    except ValueError:
        raise ValueError(
            "Invalid argument value: date_from must be in format of YYYY-MM-DD. Also check if it is a valid date."
        )
    except TypeError:
        raise TypeError(
            "Invalid argument type: date_from must be in string format of YYYY-MM-DD."
        )

    try:
        date_to != datetime.strptime(date_to, "%Y-%m-%d").strftime("%Y-%m-%d")
        #            raise ValueError
    except ValueError:
        raise ValueError(
            "Invalid argument value: date_to must be in format of YYYY-MM-DD. Also check if it is a valid date."
        )
    except TypeError:
        raise TypeError(
            "Invalid argument type: date_to must be in string format of YYYY-MM-DD."
        )

    error_msg = (
        "Invalid values: date_from should be smaller or equal"
        " to date_to (or today's date if date_to is not specified)."
    )

    if pd.to_datetime(date_to) < pd.to_datetime(date_from):
        raise ValueError(
            error_msg,
        )
    if pd.to_datetime(date_to) > pd.to_datetime("today").normalize():
        raise ValueError("Invalid values: date_to should be smaller or equal to today.")

    if location is not None:

        if not (isinstance(location, list)):
            raise TypeError(
                "Invalid argument type: location must be a list of strings."
            )

        for item in location:
            if not (isinstance(item, str)):
                raise TypeError(
                    "Invalid argument type: values inside location list must be a strings."
                )

        query += " and location in @location"

    try:
        covid_df = pd.read_csv(url, parse_dates=["date"])
    except BaseException:
        return "The link to the data is broken."

    covid_df = covid_df.query(query)
    covid_df = covid_df[~covid_df["iso_code"].str.startswith("OWID")]
    return covid_df



# File Name: plot_spec.py
# Author: Rong Li

import pandas as pd
from datetime import datetime
import altair as alt
from dateutil.parser import parse


def plot_spec(
    df, location=["Canada"], val="new_cases", date_from=None, date_to=None, title=None
):
    """
    Create a line chart presenting specific country/countries COVID information
    within a time period

    Parameters
    ----------
    df  : Pandas dataframe
        Pandas dataframe of the selected covid data from get_data()
    location : list, optional
        List of target country names or . By default ["Canada"]
    val : str, optional
        Quantitative values of interests. Must be numeric variable.
        Also known as a 'measure'. By default 'new_cases'
    date_from : str, optional
        Start date of the data range with format in "YYYY-MM-DD" format. By
        default 'None' is used to represent 7 days prior to today's date
    date_to : str, optional
        End date of data range with format in "YYYY-MM-DD" format. By default
        'None' is used to represent today's date
    title : str, optional
        The title of the plot. By default 'None' will be generated based on val

    Returns
    -------
    plot
        Altair line chart created

    Examples
    --------
    >>> plot_spec(df, location=["Canada", "Turkey"], val="new_cases", date_from="2022-01-01", date_to="2022-01-07")

    """
    # init dates if None
    if date_from is None:
        date_from = (
            pd.to_datetime("today").normalize() - pd.to_timedelta(7, unit="d")
        ).strftime("%Y-%m-%d")

    if date_to is None:
        date_to = pd.to_datetime("today").normalize().strftime("%Y-%m-%d")

    # Exception Handling
    if not isinstance(df, pd.DataFrame):
        raise FileNotFoundError("Data not found. There may be a problem with data URL.")

    if not isinstance(location, list):
        raise TypeError("Invalid argument type: location must be a list of strings.")
    for item in location:
        if not (isinstance(item, str)):
            raise TypeError(
                "Invalid argument type: values inside location list must be strings."
            )

    if not isinstance(val, str):
        raise TypeError("Invalid argument type: val must be a string.")

    if df[val].dtypes.kind == "O":
        raise TypeError("Invalid argument type: val must be a numeric variable.")

    try:
        date_from != datetime.strptime(date_from, "%Y-%m-%d").strftime("%Y-%m-%d")
    except ValueError:
        raise ValueError(
            "Invalid argument value: date_from must be in format of YYYY-MM-DD. Also check if it is a valid date."
        )

    try:
        date_to != datetime.strptime(date_to, "%Y-%m-%d").strftime("%Y-%m-%d")
    except ValueError:
        raise ValueError(
            "Invalid argument value: date_to must be in format of YYYY-MM-DD. Also check if it is a valid date."
        )

    if pd.to_datetime(date_to) < pd.to_datetime(date_from):
        raise ValueError(
            "Invalid values: date_from should be smaller or equal to date_to (or today's date if date_to is not specified)."
        )
    if pd.to_datetime(date_to) > pd.to_datetime("today").normalize():
        raise ValueError("Invalid values: date_to should be smaller or equal to today.")

    if title is not None:
        if not isinstance(title, str):
            raise TypeError("Invalid argument type: title must be a string.")

    # Parse date
    date_from = parse(date_from)
    date_to = parse(date_to)

    # Convert 'date' to date format
    df["date"] = pd.to_datetime(df["date"])

    # Filter by date
    df = df.query("date >= @date_from & date <= @date_to")

    # Filter by country
    df = df.query("location in @location")

    # Remove aggregated locations
    df = df[~df["iso_code"].str.startswith("OWID")]

    # Create Y axis label
    val_label = val.replace("_", " ").title()

    # init plot title if None
    if title is None:
        title = f"COVID-19 {val_label}"

    # Create line plot
    line = (
        alt.Chart(df, title=title)
        .mark_line()
        .encode(
            x=alt.X(
                "yearmonthdate(date):T", axis=alt.Axis(format="%e %b, %Y"), title="Date"
            ),
            y=alt.Y(val, title=val_label),
            color=alt.Color("location", legend=None),
            tooltip=["location", val],
        )
    )

    # Use direct labels
    order = df.loc[df["date"] == df["date"].max()].sort_values(val, ascending=False)

    text = (
        alt.Chart(order)
        .mark_text(dx=20)
        .encode(
            x=alt.X(
                "yearmonthdate(date):T", axis=alt.Axis(format="%e %b, %Y"), title="Date"
            ),
            y=alt.Y(val, title=val_label),
            text="location",
            color="location",
        )
    )
    plot = line + text
    return plot



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['covizpy']

package_data = \
{'': ['*']}

install_requires = \
['altair-saver>=0.5.0,<0.6.0', 'altair>=4.2.0,<5.0.0', 'pandas>=1.3.5,<2.0.0']

setup_kwargs = {
    'name': 'covizpy',
    'version': '1.0.24',
    'description': 'Provides access to Covid-19 data from Our World in Data, and functions to generate relevant charts and summaries.',
    'long_description': '# covizpy <img src=\'img/logo.png\' align="right" height="139" />\n\n[![codecov](https://codecov.io/gh/ubc-mds/covizpy/branch/main/graph/badge.svg?token=bCRbU4Axjx)](https://codecov.io/gh/ubc-mds/covizpy)\n[![ci-cd](https://github.com/UBC-MDS/covizpy/actions/workflows/ci-cd.yml/badge.svg)](https://github.com/UBC-MDS/covizpy/actions/workflows/ci-cd.yml)\n[![Documentation Status](https://readthedocs.org/projects/covizpy/badge/?version=latest)](https://covizpy.readthedocs.io/en/latest/?badge=latest)\n\n`covizpy` is a Python package that provides easy access to Covid-19 data from [Our World in Data](https://ourworldindata.org/coronavirus), as well as functions to generate relevant Covid-19 charts and summaries easily. We aim to make `covizpy` simple and easy to use. Our goal is to enable anyone with basic Python programming knowledge to access and visualize Covid-19 data, and make their own informed decisions and conclusions.\n\nThere are existing Python packages that allow users to download and generate Covid-19 charts. For example, [covid19pandas](https://github.com/PayneLab/covid19pandas) is a package that presents COVID-19 data from Johns Hopkins University and The New York Times in pandas dataframes, to make analysis and visualization easier in a Python environment.\n\nWhile other packages have more advanced plotting capabilities, we provide simpler functions that allow users to answer questions regarding the Covid-19 pandemic as quickly as possible.\n\n## Features\n\nThis package contains four functions: `plot_metric`, `plot_spec`, `get_data` and `plot_summary`.\n\n* `plot_metric`: Create a line chart presenting COVID total new cases versus another metric within a time period\n\n* `plot_spec`: Create a line chart presenting specific country/countries COVID information within a time period\n\n* `get_data`: User can retrieve the COVID data from the source as a pandas dataframe. Specific data can be retrieved by passing the date range and the list of countries\n\n* `plot_summary`: Create a horizontal bar chart summarising a specified variable and value within a time period\n\n## Dependencies\n\nBefore installing the package, following packages must be installed:\n\n- python = "^3.9"\n- pandas = "^1.3.5"\n- altair = "^4.2.0"\n- altair-saver = "^0.5.0"\n\n## Installation\n\n```bash\npip install covizpy\n```\n\n## Usage and Examples \n\nTo use the package, import the package with following commands:\n\n```python\nfrom covizpy.get_data import get_data\nfrom covizpy.plot_summary import plot_summary\nfrom covizpy.plot_metric import plot_metric\nfrom covizpy.plot_spec import plot_spec\n```\n\nTo use the functions, see below examples:\n\n### Retrieve COVID-19 data with specified date range and default all locations\n\n```python\ndf = get_data(date_from="2022-01-01", date_to="2022-01-21")\n```\n\n### Plot summary graph (bar chart)\n\n```python\nplot_summary(df, var="location", val="new_cases", fun="sum", date_from="2022-01-01", date_to="2022-01-15", top_n=10)\n```\n\n![Summary graph](https://github.com/UBC-MDS/covizpy/raw/main/img/plot_summary.png)\n\n### Plot COVID-19 cases for specific countries (line chart)\n\n```python\nplot_spec(df, location=["Canada", "Turkey"], val="new_cases", date_from="2022-01-01", date_to="2022-01-07")\n```\n\n![New COVID-19 specific graph](https://github.com/UBC-MDS/covizpy/raw/main/img/plot_spec.png)\n\n### Plot new COVID-19 cases versus another metric (line chart)\n\n```python\nplot_metric(location="Canada", metric="positive_rate", date_from="2022-01-01", date_to="2022-01-15")\n```\n\n![New COVID-19 case metric graph](https://github.com/UBC-MDS/covizpy/raw/main/img/plot_metric.png)\n\n\n## Contributors\n\n* Rohit Rawat\n* Rong Li\n* Thomas Siu\n* Ting Zhe Yan\n\n## Contributing\n\nInterested in contributing? Check out the contributing guidelines. Please note that this project is released with a Code of Conduct. By contributing to this project, you agree to abide by its terms.\n\n## License\n\n`covizpy` was created by Rohit Rawat, Rong Li, Thomas Siu, Ting Zhe Yan . It is licensed under the terms of the MIT license.\n\n## Credits\n\n`covizpy` was created with [`cookiecutter`](https://cookiecutter.readthedocs.io/en/latest/) and the `py-pkgs-cookiecutter` [template](https://github.com/py-pkgs/py-pkgs-cookiecutter).\n',
    'author': 'Rohit Rawat, Rong Li, Thomas Siu, Ting Zhe Yan',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/UBC-MDS/covizpy',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



# File Name: plot_metric.py
# Author: Rohit Rawat

import altair as alt
from covizpy.get_data import get_data

alt.data_transformers.enable("data_server")


def plot_metric(
    location="Canada", metric="positive_rate", date_from=None, date_to=None
):
    """
    Create a line chart visualizing COVID total new
    cases and another metric for a specific time period

    Parameters
    ----------
    location  : str, optional
                The name of the location to be plotted with the new COVID cases.
                It can be any of the values present in the location column of df
    metric    : str, optional
                The name of the metric to be plotted with the new COVID cases.
                It can be one of the these: "reproduction_rate", "positive_rate",
                or any other numeric column
    date_from : str, optional
                Start date of the plot in "YYYY-MM-DD" format, by default "2022-01-01"
    date_to   : str, optional
                End date of the plot in "YYYY-MM-DD" format, by default "2022-01-13"
    Returns
    -------
    chart
        The line chart created
    """

    # Check the input format of arguments
    if not isinstance(metric, str):
        raise TypeError("Incorrect argument type: Metric 1 input should be a string")

    if (not isinstance(date_from, str)) and date_from is not None:
        raise TypeError(
            "Incorrect argument type: The starting date should be in string format"
        )

    if (not isinstance(date_to, str)) and date_to is not None:
        raise TypeError(
            "Incorrect argument type: The end date should be in string format"
        )

    # Check if func it is able to fetch the data
    try:
        df = get_data(date_from, date_to)
    except FileNotFoundError:
        raise FileNotFoundError(
            "Data not found! There may be a problem with data URL or your date format."
        )

    # Check whether the location is in the df or not
    if not isinstance(location, str):
        raise TypeError("Invalid argument type: location must be a single string.")

    if not (location in df.location.unique()):
        raise ValueError(
            "Invalid argument value: location is not present in available locations."
        )

    # Check if the metric provided is present in the data frame or not
    if metric not in df.columns:
        raise ValueError(
            "Incorrect argument value: The metric chosen is not one of the columns in dataframe"
        )

    df = df.query("location in @location")

    metric_label = "Mean " + metric.replace("_", " ")
    title_label = "Daily COVID cases versus " + metric_label + " in " + location

    base = alt.Chart(df).encode(
        x=alt.X(
            "yearmonthdate(date):T", axis=alt.Axis(format="%e %b, %Y"), title="Date"
        )
    )

    line1 = base.mark_line(color="skyblue", interpolate="monotone").encode(
        alt.Y(
            "sum(new_cases)",
            scale=alt.Scale(zero=False),
            axis=alt.Axis(title="Daily new cases", titleColor="skyblue"),
        )
    )

    line2 = base.mark_line(color="orange", interpolate="monotone").encode(
        alt.Y(
            f"mean({metric})",
            scale=alt.Scale(zero=False),
            axis=alt.Axis(title=metric_label, titleColor="orange"),
        )
    )

    plot = alt.layer(line1, line2, title=title_label).resolve_scale(y="independent")

    return plot
